function mask_emission_lines_SAURON,npix,Vsys,emission_setup,velscale,l0_gal,lstep_gal,sigma
; Return a list of goodpixels to fit allowing to exclude regions
; potentially affected by gas emission. Wavelength values are assumed
; to be ln-rebinned, and are defined by the l0_gal,lstep_gal,npix
; parameters. The position of gas emission lines is set by the input
; emission_setup structure and the width of the mask by the sigma
; parameter.

c = 299792.458d
msigma = sigma/velscale

goodpixels = range(0,npix-1)
tmppixels  = goodpixels

; looping over the listed emission-lines and mask those tagged with an
; 'm' for mask.
for i = 0,n_elements(emission_setup.i)-1 do begin
    if (emission_setup.action[i] eq 'm') then begin
        print,'--> masking ' + emission_setup.name[i]
        meml_cpix = ceil((alog(emission_setup.lambda[i])-l0_gal)/lstep_gal+Vsys/velscale)
        meml_bpix = meml_cpix - msigma
        meml_rpix = meml_cpix + msigma
        w = where(goodpixels ge meml_bpix and goodpixels le meml_rpix) & tmppixels[w] = -1
    endif
endfor
w = where(tmppixels ne -1)
goodpixels = goodpixels[w]

return,goodpixels
end

function remouve_detected_emission_SAURON,galaxy,bestfit,emission_templates,sol_gas_A,$
                                   velscale,l0_gal,lstep_gal,ynunc=ynunc,Vsys=Vsys,$
                                   AoN=AoN,goodpixels=goodpixels
; Given the galaxy spectrum, the best fit, the emission-line
; amplitudes, and the array containing the best-fitting emission-line
; templates (returned by GANDALF), this function simply compute the
; residual-noise level, compares it to the amplitude of each lines,
; and remouves from the galaxy spectrum only the best-matching
; emission-line templates for which the correspoding amplitude over
; noise ratio A/N exceed the thresholds and conditions set in Sarzi et
; al. (2005).
;
; A list of goodpixels may be optionally input, for instance if any
; pixel was excluded by sigma-clipping during the continuum and
; emission-line fitting.
;
; Also outputs the computed A/N ratios
;
; Example of use   
;  - std case -
; neat_spec = remouve_detected_emission_SAURON(galaxy_spec[*,i],bestfit_spec,emission_templates,sol_A,$
;                                              velscale,l0_gal,lstep_gal,ynunc=0,Vsys=Vsys,$
;                                              AoN=sol_AoN,goodpixels=goodpixels)
;
; - unc case -
; neat_spec = remouve_detected_emission_SAURON(galaxy_spec[*,i],bestfit_spec,emission_templates,sol_A,$
;                                              velscale,l0_gal,lstep_gal,ynunc=1,Vsys=Vsys$
;                                              AoN=sol_AoN,goodpixels=goodpixels)
;
;
; if ynunc and Vsys are not specified they will both taken as 0

; Get overall Residual Noise lvl.
resid = galaxy - bestfit
if keyword_set(goodpixels) then resid = resid[goodpixels]
resid_noise = robust_sigma(resid, /ZERO)
; A/N of Hb and [OIII] line
AoN = sol_gas_A/resid_noise
; A/N of [NI]. Take Noise lvl in the Mgb region and compare it not
; with the amplitude of [NI]5200 but with the max of the combined 
; [NI] lines
Mgbpix1 = ceil((alog(5160.00)-l0_gal)/lstep_gal+Vsys/velscale)
Mgbpix2 = ceil((alog(5185.00)-l0_gal)/lstep_gal+Vsys/velscale)
ind_Mgb = indgen(Mgbpix2-Mgbpix1+1) + Mgbpix1
resid_noise_Mgb = robust_sigma(resid[ind_Mgb], /ZERO)
AoN[2] = max(emission_templates[*,2])/resid_noise_Mgb

; Create neat spectrum, that is, a spectrum where only detected
; emission has been remouved, according to Sarzi et al. (2005)
neat_galaxy = galaxy
; "Standard" constrained-fit case. Fit first [OIII] (while masking Hb
; and [NI]) then impose the [OIII] kinematics to fit Hb and [NI].
; Hb is detected if [OIII] is detected and [NI] is detected if
; both Hb and [OIII] are detected
if (ynunc eq 0) then begin
    AoN_thresholds = [3.0,4.0,4.0]
    if (AoN[1] ge AoN_thresholds[1]) then begin
        ; [OIII] detected! Hb and [NI] still could not...
        neat_galaxy = neat_galaxy - emission_templates[*,1]
        if (AoN[0] ge AoN_thresholds[0]) then begin
            ; Hb detected! [NI] still could not...
            neat_galaxy = neat_galaxy - emission_templates[*,0]
            if (AoN[2] ge AoN_thresholds[2]) then begin
                ; [NI] detected!
                neat_galaxy = neat_galaxy - emission_templates[*,2]            
            endif
        endif
    endif
endif
; Unconstrained-fit case. Fit first [OIII] and Hb independently (twice
; and while masking [NI]), then impose the Hb kinematics on [NI]
; [NI] is detected only if both Hb and [OIII] are detected
if (ynunc eq 1) then begin
    AoN_thresholds = [5.0,4.0,4.0]
    if (AoN[0] ge AoN_thresholds[0]) then begin
        ; Hb detected!
        neat_galaxy = neat_galaxy - emission_templates[*,0]
    endif
    if (AoN[1] ge AoN_thresholds[1]) then begin
        ; [OIII] detected!
        neat_galaxy = neat_galaxy - emission_templates[*,1]
    endif
    if (AoN[0] ge AoN_thresholds[0] and AoN[1] ge AoN_thresholds[1] and AoN[2] ge AoN_thresholds[2]) then begin
        ; [NI] detected! Also [OIII] and Hb needed to
        neat_galaxy = neat_galaxy - emission_templates[*,2]            
    endif
endif

return,neat_galaxy
end

pro pause
in=' '
READ,"Press enter",in
end

PRO GANDALF_SAURON_STD, INFILE=infile, EMISSION_SETUP_FILE=emission_setup_file, LIBFILE=libfile, $
                        MDEGREE=mdegree, OUTFILE=outfile, VSYS=vsys, PLOT=plot, DEBUG=debug,$
                        FOR_ERRORS=for_errors
; Example of an IDL wrapper calling PPXF and GANDALF in order to
; derive the stellar and gaseous kinematics in the case of SAURON
; spectra. The stellar continuum is matched with a combination of
; stellar spectra and single age stellar population models, whereas
; emission-lines are represented by Gaussian templates. 
;
; In this case the kinematics of the [OIII] doublet lines is measured
; first, and then it is imposed on the Hb and [NI] lines while the
; fluxes of these lines are subsequently derived.

IF KEYWORD_SET(plot) THEN window,0,xsize=500,ysize=500, retain=2
IF NOT KEYWORD_SET(FOR_ERRORS) then QUIET=1

c = 299792.4580d ; Speed of light in km/s
velscale = 60.0d ; km/s SAURON velocity scale
int_disp =108.0d ; SAURON instrumental resolution

; Preliminary Checks !
IF NOT KEYWORD_SET(INFILE)              THEN message,'Please enter the filename of the input spectrum'
IF NOT KEYWORD_SET(EMISSION_SETUP_FILE) THEN message,'Please enter the filename with the emission-line setup'
IF NOT KEYWORD_SET(LIBFILE)             THEN message,'Please enter the filename with the list of templates'
IF NOT KEYWORD_SET(OUTFILE)             THEN message,'Please enter the filename of the output file'
IF NOT KEYWORD_SET(MDEGREE)             THEN mdegree=0

; A) Reading one SAURON ln-rebinned spectrum
; This can be easily modified to read in a data cube with n_spec
; spectra, instead of having just one spectrum

print,'--> Reading input spectrum ... '+infile
galaxy = readfits(infile,hdr_gal,/SILENT)
s=size(galaxy) 
n_pix  = s[1]
n_spec = 1
; Galaxy spectrum
galaxy_spec = galaxy
; Error spectrum. For now give everything the same weight
galaxy_err  = galaxy*0.+1.

; B) Reading the templates, which are also already ln-rebinned
print,'--> Reading template library ...'+libfile
readcol, libfile, dummy, dir, SKIPLINE=1, NUMLINE=1, FORMAT='(A,A)', COMMENT='#', /SILENT
readcol, libfile, templ_name, templ_file, templ_vel, templ_sig, SKIPLINE=2, FORMAT='(A,A,F,F)', COMMENT='#', /SILENT
;
n_templ=n_elements(templ_name)
print, strtrim(string(n_elements(templ_name)),2)+'  templates found'

; C) Homogenising the template library
;
; Check first if they all are in ln-lambda steps
; and fill in the template properties array.
templ_props=dblarr(3,n_templ)
FOR i=0, n_templ-1 DO BEGIN
    fname=dir+'/'+templ_file[i]
    dummy=mrdfits(fname[0],0,hdr_templ, /silent)
    templ_props[*,i]=[sxpar(hdr_templ,'NAXIS1',/SILENT),sxpar(hdr_templ,'CRVAL1',/SILENT),sxpar(hdr_templ,'CDELT1',/SILENT)]
    IF i GE 1 THEN BEGIN
    	IF templ_props[2,i] NE templ_props[2,i-1] THEN BEGIN
	    print, 'STOP: All the templates must have the same log-lambda step'
	    stop
	ENDIF
    ENDIF
ENDFOR
; Which you can then use to homogenise the template library
min_npix=min(templ_props[0,*])
max_lnlam=max(templ_props[1,*])
templates=dblarr(min_npix, n_templ)
; which will include also accounting for difference between the
; instrumental resolution of SAURON and the Vazdekis library.
sigma = sqrt(int_disp^2-44.0d^2)/velscale ; Quadratic sigma difference in pixels Vazdekis --> SAURON
;
print,'--> Convolving template library to SAURON resolution ...'
FOR i=0, n_templ-1 DO BEGIN
    fname=dir+'/'+templ_file[i]
    tmp=mrdfits(fname[0],0,hdr_templ, /silent)
    pix0=floor((max_lnlam-templ_props[1,i])/templ_props[2,i])
    lsf = psf_Gaussian(NPIXEL=10.0*sigma, FWHM=2.35*sigma, /NORM, NDIM=1)
    templates[*,i]=convol(tmp[pix0:pix0+min_npix-1],lsf) ; Degrade template to SAURON resolution
ENDFOR

; D) Computing the velocity offset between the starting wavelengths
; of the galaxy and template spectra. 
print,'--> Set the template-galaxy spectra offset'
l0_gal   = sxpar(hdr_gal,'CRVALS')
l0_templ = max_lnlam
offset = -(l0_gal-l0_templ)*c

; E) Preamble for Ppxf
print,'--> ... and the initial guesses for ppxf using the input recession velocity'
lstep_gal = sxpar(hdr_gal,'CDELTS')
start = [Vsys+offset,50.0d]

; F) Read in emission-line setup file and fill in emission-line structure
print,'--> Reading in the emission-line setup file, and creating the corresponding structure'
eml_file = emission_setup_file
readcol,eml_file,eml_i,eml_name,eml_lambda,eml_action,eml_kind,eml_a,eml_v,eml_s,eml_fit,$
  f='(i,a,f,a,a,f,f,f,a)',skipline=2,comment='#',/silent
; creating emission setup structure, to be used for masking the
; emission-line contaminated regions and fitting the emission lines in
; ppxf_gas
emission_setup = create_struct('i',eml_i,'name',eml_name,'lambda',eml_lambda,'action',eml_action,$
                               'kind',eml_kind,'a',eml_a,'v',eml_v,'s',eml_s,'fit',eml_fit)
; setting the galaxy systemic velocity and 50km/s as initial guesses
; for the gas velocities and velocity dispersions, over-writing the
; emission-line setup values
emission_setup.v = Vsys & emission_setup.s = 50.0d

; G) Create all the arrays that will contain both the best-fitting
; parameters and output spectra.
print,'--> Creating all arrays that will contain all best fitting parameters and output spectra'
; stellar kinematics  (V,sig,h3,h4)
sol_star         = dblarr(4,n_spec)
; gaseous kinematics 
n_lines          = n_elements(where(emission_setup.kind eq 'l'))
sol_gas_F        = dblarr(n_lines,n_spec)
sol_gas_A        = dblarr(n_lines,n_spec)
sol_gas_V        = dblarr(n_lines,n_spec)
sol_gas_S        = dblarr(n_lines,n_spec) 
sol_gas_AoN      = dblarr(n_lines,n_spec)
; normalised template weight
sol_nweights     = dblarr(n_templ,n_spec)
; output spectra
galaxy_spectra   = dblarr(n_pix,n_spec)
bestfit_spectra  = dblarr(n_pix,n_spec)
emission_spectra = dblarr(n_pix,n_spec)
neat_spectra     = dblarr(n_pix,n_spec)
; output optimal template
n_pix_templ      = n_elements(templates[*,0])
otempl_spectra   = dblarr(n_pix_templ,n_spec)

; H) Start to loop over all the spectra in the cubes and fit the
; stellar and gaseous kinematics
print,'--> Beginning to loop all spectra in the SAURON cube'
FOR i=0,n_spec-1 DO BEGIN

    print, '==> BIN NUMBER:   ',strtrim(string(i+1),1),FORMAT='($,A24,I5)'
    print,''

    ; 1] Masking prior to ppxf
    ;
    ; Call mask_emission_lines_SAURON, which loops over the listed
    ; emission-lines and mask those tagged with an 'm' for mask. Use a
    ; 500 km/s sigma mask
    print,'--> Mask regions potentially affected by gas emission!'
    sig_mask = 500.0
    goodpixels = mask_emission_lines_SAURON(n_pix,Vsys,emission_setup,velscale,l0_gal,lstep_gal,sig_mask)
 
    ; 2] PPXF fit to derive the stellar kinematics!
    ;
    ; This is consistent with the values presented in Emsellem et
    ; al. 2004 (MNRAS 352 721) in that an additive constant is used
    ; together with multiplicative polynomials
    loadct,12,/silent
    PPXF, templates, galaxy_spec[*,i], galaxy_err[*,i], velscale, start, sol,$
      goodpixels=goodpixels,plot=plot, moments=4, degree=0, mdegree=mdegree
    print,'--> Fit to the stellar continuum masking regions potentially affected by gas emission!'
    if keyword_set(DEBUG) then pause
    star_kin = sol

    ; 3] Lift the mask in the [OIII] regions
    print,'--> Lift mask in [OIII] region'
    i_o3 = where(emission_setup.name eq '[OIII]')
    emission_setup.action[i_o3] = 'f'
    goodpixels = mask_emission_lines_SAURON(n_pix,Vsys,emission_setup,velscale,l0_gal,lstep_gal,sig_mask)

    ; 4] GANDALF fit for the [OIII] kinematics
    ;
    sol = star_kin
    GANDALF, templates, galaxy_spec[*,i], galaxy_err[*,i], velscale, sol, emission_setup, $
      l0_gal, lstep_gal, GOODPIXELS=goodpixels, MDEGREE=mdegree, INT_DISP=int_disp, $
      BESTFIT=bestfit_spec, EMISSION_TEMPLATES=emission_templates, WEIGHTS=weights, $
      PLOT=plot,QUIET=quiet,FOR_ERRORS=for_errors

    print,'--> ... and fit simultaneously the stellar continuum and the [OIII] emission lines!'
    if keyword_set(DEBUG) then pause

    ; 5] Lift the mask also in the Hb and [NI] region, and manipulate the
    ;    emission_setup in order to hold the gas kinematics to the [OIII]
    ;    kinematics we just derived
    print,'--> Lift the any remaining emission-line mask'
    i_mlines = where(emission_setup.action eq 'm')
    emission_setup.action[i_mlines] = 'f'
    goodpixels = mask_emission_lines_SAURON(n_pix,Vsys,emission_setup,velscale,l0_gal,lstep_gal,sig_mask)
    emission_setup.v          = sol[2]
    emission_setup.s          = sol[3]
    emission_setup.fit[i_o3]  = 'h'

    ; 6] GANDALF fit for the all lines, but hold the gas kinematics to
    ;    the [OIII]
    ;
    sol = star_kin
    GANDALF, templates, galaxy_spec[*,i], galaxy_err[*,i], velscale, sol, emission_setup, $
      l0_gal, lstep_gal, GOODPIXELS=goodpixels, MDEGREE=mdegree, INT_DISP=int_disp, $
      BESTFIT=bestfit_spec, EMISSION_TEMPLATES=emission_templates, WEIGHTS=weights, $
      PLOT=plot,QUIET=quiet,FOR_ERRORS=for_errors
    print,'--> ... and fit simultaneously the stellar continuum and all emission lines! (with [OIII] kinematics)'
    if keyword_set(DEBUG) then pause

    ; 7] Make unconvolved optimal template and galaxy spectrum cleaned
    ;    by detected emission. The last procedure is consistent with
    ;    the procedure of paper V.
    nweights = weights[0:n_templ-1]/total(weights[0:n_templ-1])
    otempl_spec = dblarr(n_pix_templ)
    for j=0,n_templ-1 do otempl_spec = otempl_spec + templates[*,j]*nweights[j]
    print,'--> Creating the unconvolved optimal template spectrum...'
    ; Best fitting amplitudes
    sol_A = sol[dindgen(n_lines)*4+1]
    ; Remouve detected emission, following Sarzi et al. (2005)
    neat_spec = remouve_detected_emission_SAURON(galaxy_spec[*,i],bestfit_spec,emission_templates,sol_A,$
                                                 velscale,l0_gal,lstep_gal,ynunc=0,Vsys=Vsys,$
                                                 AoN=sol_AoN,goodpixels=goodpixels)
    print,'--> ... and cleaning the galaxy spectrum from any detected gas emission line'

    ; 8] Store all results
    ; stellar kinematics  (V,sig,h3,h4)
    star_kin[0]           = star_kin[0] - offset
    sol_star[*,i]         = star_kin[0:3]
    ; gaseous kinematics 
    sol_gas_F[*,i]        = sol[dindgen(n_lines)*4+0]
    sol_gas_A[*,i]        = sol[dindgen(n_lines)*4+1]
    sol_gas_V[*,i]        = sol[dindgen(n_lines)*4+2]
    sol_gas_S[*,i]        = sol[dindgen(n_lines)*4+3]
    sol_gas_AoN[*,i]      = sol_AoN
    ; normalised template weight
    sol_nweights[*,i]     = nweights
    ; output spectra
    galaxy_spectra[*,i]   = galaxy_spec[*,i]
    bestfit_spectra[*,i]  = bestfit_spec
    emission_spectra[*,i] = total(emission_templates,2)
    neat_spectra[*,i]     = neat_spec
    ; output optimal template
    otempl_spectra[*,i]   = otempl_spec
    print,'--> Storing best fitting paramaters and output spectra ...'
    
    ; 9] Re-mask all emission-line regions, reset initial guesses for
    ;    the gas kinematics, and let [OIII] free to vary again
    emission_setup.action = 'm'
    emission_setup.v      = Vsys 
    emission_setup.s      = 50.0d
    emission_setup.fit[i_o3] = 'f'
    print,'--> ... and reset emission-line setup structure for fitting the next spectrum'
ENDFOR

; I) Write output files

print,'--> Finally, save the results...'

sol_gas = dblarr(6+n_lines*5,n_spec)
for i = 0,n_spec-1 do begin
    sol_gas[0:5,i] = [sol_star[*,i],0.,0.]
    sol_gas[indgen(n_lines)*5+6+0,i] = sol_gas_F[*,i]
    sol_gas[indgen(n_lines)*5+6+1,i] = sol_gas_A[*,i]
    sol_gas[indgen(n_lines)*5+6+2,i] = sol_gas_V[*,i]
    sol_gas[indgen(n_lines)*5+6+3,i] = sol_gas_S[*,i]
    sol_gas[indgen(n_lines)*5+6+4,i] = sol_gas_AoN[*,i]
endfor

writefits,outfile,galaxy_spectra          ; input spectrum         
writefits,outfile,bestfit_spectra, /append; best-fitting model
writefits,outfile,emission_spectra,/append; best-fitting emission-line spectrum
writefits,outfile,neat_spectra,    /append; input spectrum cleaned by detected emission
writefits,outfile,sol_gas,         /append; stellar and gas kinematics best parameters
writefits,outfile,sol_nweights,    /append; templates normalised weights
writefits,outfile,otempl_spectra,  /append; optimal stellar template

end


